//
//  Stencil.cpp
//  MANITEST
//
//  Created by Alhussein Fawzi on 15.06.15.
//  Copyright (c) 2015 Alhussein Fawzi. All rights reserved.
//

#include "Stencil.h"

Stencil::Stencil() {
    // Set the default minPoint and maxPoint
    int i; int MAX_GRP_SIZE = 10;
    for (i=0; i<MAX_GRP_SIZE; i++) {
        minPoint.push_back(-INFINITY);
        maxPoint.push_back(INFINITY);
    }
}
    
bool Stencil::checkOutOfBounds(point param) {
    int i;
    for (i = 0; i<param.size(); i++) {
        if (this->offsets[i]+param[i]*hs[i] > maxPoint[i] + 1e-8 || this->offsets[i]+param[i]*hs[i] < minPoint[i] - 1e-8) {
            return false;
        }
    }
    return true;
}

cv::Mat Stencil::computeMetric(std::vector <double> para) {
    // Output the metric
    cv::Mat J = parentClass->trans->jacobi(para);
    cv::Mat M = (J * J.t());
    return M;
}

std::vector<double> Stencil::coords2Para(std::vector<int>* coords) {
    std::vector <double> para((*coords).size());
    int i;
    for (i=0; i<(*coords).size();i++) {
        para[i] = offsets[i] + (*coords)[i] * hs[i];
    }
    return para;
}

// Return its new id
int Stencil::initializeNode(point node) {
    
    int id_new;
    std::pair<std::map<point,int>::iterator,bool> ret_insert;
    std::vector <int> ids_rStencil;
    ret_insert = parentClass->coords2Id.insert ( std::pair<point ,int>(node,parentClass->id2Coords.size()) );
    // Insert curpoint in the map
    if (ret_insert.second == true) {
        id_new = (int) parentClass->id2Coords.size();
        
        // Node does not exist; add it in id2Coords (already added to coords2Id up)
        parentClass->id2Coords.push_back(node);
        parentClass->dist.push_back(INFINITY);
        parentClass->visited.push_back(false);
        parentClass->metrics.push_back(cv::Mat(0, 0, CV_32F)); // Set empty matrix
        return id_new;
    }
    else {
        // Otherwise, it is already there, just add it to the set (if it is not an already visited node)
        return (ret_insert.first)->second;
    }
    
}